/* -----------------------------------------------------------------------------
 * Copyright (c) 2013-2015 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        05. June 2015
 * $Revision:    V1.2
 *
 * Project:      Ethernet Media Access (MAC) Definitions for Freescale MK64F
 * -------------------------------------------------------------------------- */

#ifndef __EMAC_MK64F_H
#define __EMAC_MK64F_H

#include <string.h>

#include "Driver_ETH_MAC.h"
#include "RTE_Device.h"

#include "fsl_device_registers.h"
#include "fsl_osc_hal.h"
#include "fsl_sim_hal.h"
#include "fsl_enet_hal.h"

#if (RTE_ENET == 0)
  #error "ENET peripheral not configured in RTE_Device.h!"
#endif

#if ((RTE_ENET_MII_MODE == 0) && (RTE_ENET_MII_CLOCK_SOURCE != 0))
  #error "ENET peripheral in MII mode must use MII0_TXCLK/MII0_RXCLK as a clock source."
#endif

#if ((RTE_ENET_MII_MODE == 1) && (RTE_ENET_MII_CLOCK_SOURCE == 0))
  #error "ENET peripheral in RMII mode must use OSCERCLK or ENET_1588_CLKIN as a clock source."
#endif

#if (RTE_ENET_MII_MODE == 1)
  #if (RTE_ENET_MII_CLOCK_SOURCE == 1)
    #define ENET_RMII_CLOCK_SOURCE      kClockRmiiSrcExtalClk
    #define ENET_OSCERCLK_ENABLE        1
  #else
    #define ENET_RMII_CLOCK_SOURCE      kClockRmiiSrcExt
  #endif
#endif


/* EMAC Driver state flags */
#define EMAC_FLAG_INIT     (1U << 0)        // Driver initialized
#define EMAC_FLAG_POWER    (1U << 1)        // Driver power on


/* EMAC Driver Control Information */
typedef struct _EMAC_INFO {
  ARM_ETH_MAC_SignalEvent_t cb_event;       // Event callback
  uint8_t                   flags;          // Control and state flags
  uint8_t                   tx_index;       // Transmit descriptor index
  uint8_t                   rx_index;       // Receive descriptor index
  uint8_t                   addr[6];        // Physical address
  enet_mac_config_t         cfg;            // Configuration address
  enet_bd_config            desc_cfg;       // DMA descriptor configuration
} EMAC_INFO;

#endif /* __EMAC_MK64F_H */
