/* -----------------------------------------------------------------------------
 * Copyright (c) 2013-2015 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        05. June 2015
 * $Revision:    V1.3
 *
 * Project:      I2C Driver Definitions for Freescale MK64F
 * -------------------------------------------------------------------------- */

#ifndef __I2C_MK64F_H
#define __I2C_MK64F_H

#include <string.h>

#include "Driver_I2C.h"
#include "RTE_Device.h"
#include "RTE_Components.h"

#include "fsl_device_registers.h"

#include "fsl_clock_manager.h"

#include "fsl_port_hal.h"
#include "fsl_gpio_hal.h"
#include "fsl_i2c_hal.h"

#include "fsl_edma_driver.h"

#if ((defined(RTE_Drivers_I2C0) || \
      defined(RTE_Drivers_I2C1) || \
      defined(RTE_Drivers_I2C2))   \
     && (RTE_I2C0 == 0)            \
     && (RTE_I2C1 == 0)            \
     && (RTE_I2C2 == 0))
  #error "I2C not configured in RTE_Device.h!"
#endif

/* Driver status flags */
#define I2C_INIT            (uint8_t)(1U << 0)
#define I2C_POWER           (uint8_t)(1U << 1)
#define I2C_SETUP           (uint8_t)(1U << 2)
#define I2C_DMA_RX          (uint8_t)(1U << 3)
#define I2C_DMA_TX          (uint8_t)(1U << 4)

/* Transfer control flags */
#define XFER_FLAG_START     (uint8_t)(1U << 0)
#define XFER_FLAG_RSTART    (uint8_t)(1U << 1)
#define XFER_FLAG_ADDR_SENT (uint8_t)(1U << 2)
#define XFER_FLAG_ADDR_DONE (uint8_t)(1U << 3)
#define XFER_FLAG_PENDING   (uint8_t)(1U << 4)
#define XFER_FLAG_DONE      (uint8_t)(1U << 5)
#define XFER_FLAG_DMA       (uint8_t)(1U << 6)

enum XFER_STATE {
  XFER_STATE_IDLE           =  0U,
  XFER_STATE_START          =  1U,
  XFER_STATE_RSTART         =  2U,
  XFER_STATE_STOP           =  3U,
  XFER_STATE_ADDR_NACK      =  4U,
  XFER_STATE_ADDR           =  5U,
  XFER_STATE_MASTER_RX      =  7U,
  XFER_STATE_MASTER_TX      =  8U,
  XFER_STATE_SLAVE_RX       =  9U,
  XFER_STATE_SLAVE_TX       = 10U
};

/* I2C DMA Information (Run-Time) */
typedef struct _I2C_DMA {
  dma_request_source_t    req_src;            // DMA request source
  edma_chn_state_t       *tx;                 // Tx channel info
  edma_chn_state_t       *rx;                 // Rx channel info
  edma_transfer_config_t *cfg;                // Transfer config structure
} I2C_DMA;


/* I2C Transfer Information (Run-Time) */
typedef struct _I2C_TRANSFER_INFO {
  uint32_t              addr;                 // Device address
  uint8_t volatile     *data;                 // Data pointer
  uint32_t              num;                  // Number of data to transfer
  uint32_t              cnt;                  // Data transfer counter
  uint8_t volatile      flags;                // Transfer control flags
  uint8_t volatile      state;
} I2C_TRANSFER_INFO;


/* I2C Control Information (Run-Time) */
typedef struct _I2C_CTRL {
  ARM_I2C_SignalEvent_t cb_event;             // Event Callback
  ARM_I2C_STATUS        status;               // Status flags
  I2C_TRANSFER_INFO     xfer;                 // Transfer information
  uint8_t               flags;                // Current I2C state flags
} I2C_CTRL;


/* I2C Resources definition */
typedef struct {
  I2C_Type             *base;                 // I2C peripheral base pointer
  I2C_CTRL             *ctrl;                 // Run-Time information
  IRQn_Type             irq_num;              // I2C Interrupt vector number
  sim_clock_gate_name_t clock_gate;           // I2C peripheral clock gate
  I2C_DMA              *dma;                  // DMA control block
  PORT_Type            *sda_base;             // SDA port base pointer
  uint32_t              sda_pin;              // SDA pin number
  port_mux_t            sda_mux;              // SDA pin alternate function
  PORT_Type            *scl_base;             // SCL port base pointer
  uint32_t              scl_pin;              // SCL pin number
  port_mux_t            scl_mux;              // SCL pin alternate function
} const I2C_RESOURCES;

#endif /* __I2C_MK64F_H */
