/* -----------------------------------------------------------------------------
 * Copyright (c) 2013-2015 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        05. June 2015
 * $Revision:    V1.2
 *
 * Project:      MCI Driver Definitions for Freescale MK64F
 * -------------------------------------------------------------------------- */

#ifndef __MCI_MK64F_H
#define __MCI_MK64F_H

#include "Driver_MCI.h"
#include "RTE_Device.h"

#include "fsl_device_registers.h"

#include "fsl_clock_manager.h"

#include "fsl_sim_hal.h"
#include "fsl_port_hal.h"
#include "fsl_sdhc_hal.h"

#if (defined(RTE_SDHC) && (RTE_SDHC == 0))
  #error "SDHC not configured in RTE_Device.h!"
#endif

#if ((RTE_SDHC_CD_PIN_EN != 0) || (RTE_SDHC_WP_PIN_EN != 0))
#include "fsl_gpio_driver.h"
#endif

#if   (RTE_SDHC_BUS_WIDTH_ID == 1)
  #define MCI_BUS_WIDTH_4  1U
  #define MCI_BUS_WIDTH_8  0U
#elif (RTE_SDHC_BUS_WIDTH_ID == 2)
  #define MCI_BUS_WIDTH_4  1U
  #define MCI_BUS_WIDTH_8  1U
#else
  #define MCI_BUS_WIDTH_4  0U
  #define MCI_BUS_WIDTH_8  0U
#endif

#if (RTE_SDHC_CD_PIN_EN == 0)
  #define MCI_CD_PIN       0U
#else
  #define MCI_CD_PIN       1U
#endif

#if (RTE_SDHC_WP_PIN_EN == 0)
  #define MCI_WP_PIN       0U
#else
  #define MCI_WP_PIN       1U
#endif

#define MCI_CD_PIN_ACTIVE  RTE_SDHC_CD_ACTIVE
#define MCI_WP_PIN_ACTIVE  RTE_SDHC_WP_ACTIVE


/* Peripheral reset timeout in loop cycles */
#define MCI_RESET_TIMEOUT  1000000

/* Peripheral interrupt enable bitmask */
#define SDHC_IRQ_EN_MASK  (SDHC_IRQSTATEN_CCSEN_MASK   | \
                           SDHC_IRQSTATEN_TCSEN_MASK   | \
                           SDHC_IRQSTATEN_BWRSEN_MASK  | \
                           SDHC_IRQSTATEN_BRRSEN_MASK  | \
                           SDHC_IRQSTATEN_CTOESEN_MASK | \
                           SDHC_IRQSTATEN_CCESEN_MASK  | \
                           SDHC_IRQSTATEN_CEBESEN_MASK | \
                           SDHC_IRQSTATEN_CIESEN_MASK  | \
                           SDHC_IRQSTATEN_DTOESEN_MASK | \
                           SDHC_IRQSTATEN_DCESEN_MASK  | \
                           SDHC_IRQSTATEN_DEBESEN_MASK | \
                           SDHC_IRQSTATEN_DMAESEN_MASK )

/* Peripheral interrupt clear bitmask */
#define SDHC_IRQ_CLR_MASK (0x117F01FF)


/* Driver flag definitions */
#define MCI_INIT      ((uint8_t)0x01)   /* MCI initialized           */
#define MCI_POWER     ((uint8_t)0x02)   /* MCI powered on            */
#define MCI_SETUP     ((uint8_t)0x04)   /* MCI configured            */
#define MCI_RESP_LONG ((uint8_t)0x08)   /* Long response expected    */
#define MCI_DATA_READ ((uint8_t)0x10)   /* Read transfer             */
#define MCI_DATA_MULB ((uint8_t)0x20)   /* Multiple block transfer   */

#define MCI_RESPONSE_EXPECTED_Msk (ARM_MCI_RESPONSE_SHORT      | \
                                   ARM_MCI_RESPONSE_SHORT_BUSY | \
                                   ARM_MCI_RESPONSE_LONG)

/* MCI Driver State Definition */
typedef struct MCI_Ctrl {
  ARM_MCI_SignalEvent_t cb_event;       /* Driver event callback function     */
  ARM_MCI_STATUS        status;         /* Driver status                      */
  uint32_t             *response;       /* Pointer to response buffer         */
  uint32_t              bl_sz;
  uint32_t              bl_cnt;
  uint8_t volatile      flags;          /* Driver state flags                 */
} MCI_CTRL;

#endif /* __MCI_MK64F_H */
