/* -----------------------------------------------------------------------------
 * Copyright (c) 2013-2015 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        18. June 2015
 * $Revision:    V1.2
 *
 * Project:      SAI Driver Definitions for Freescale MK64F
 * -------------------------------------------------------------------------- */

#ifndef __SAI_MK64F_H
#define __SAI_MK64F_H

#include "Driver_SAI.h"
#include "RTE_Device.h"

#include "fsl_device_registers.h"
#include "fsl_clock_manager.h"
#include "fsl_sai_hal.h"
#include "fsl_edma_driver.h"

// SAI flags
#define SAI_FLAG_INITIALIZED            (     1U)
#define SAI_FLAG_POWERED                (1U << 1)
#define SAI_FLAG_CONFIGURED             (1U << 2)


// I2S Stream Information (Run-Time)
typedef struct _I2S_STREAM_INFO {
  uint32_t                num;           // Total number of data to be transmitted/received
  uint8_t                *buf;           // Pointer to data buffer
  uint32_t                cnt;           // Number of data transmitted/receive
  uint8_t                 data_bits;     // Number of data bits
  uint32_t                protocol;      // Stream protocol
  uint8_t                 mute;          // Mute flag
} I2S_STREAM_INFO;

// SAI Status
typedef struct _SAI_STATUS {
  uint8_t tx_busy;                       // Transmitter busy flag
  uint8_t rx_busy;                       // Receiver busy flag
  uint8_t tx_underflow;                  // Transmit data underflow detected (cleared on start of next send operation)
  uint8_t rx_overflow;                   // Receive data overflow detected (cleared on start of next receive operation)
  uint8_t frame_error;                   // Sync Frame error detected (cleared on start of next send/receive operation)
} SAI_STATUS;

// SAI Information (Run-Time)
typedef struct _SAI_INFO {
  ARM_SAI_SignalEvent_t   cb_event;      // Event callback
  SAI_STATUS              status;        // Status flags
  I2S_STREAM_INFO         tx;            // Transmit information
  I2S_STREAM_INFO         rx;            // Receive information
  uint8_t                 flags;         // SAI driver flags
} SAI_INFO;

// SAI DMA
typedef struct _SAI_DMA {
  dma_request_source_t    req_src;       // DMA request source
  edma_chn_state_t       *ch_info;       // Channel info
  edma_transfer_config_t *cfg;           // Transfer configuration structure
} SAI_DMA;

// SAI Resources definitions
typedef struct {
  I2S_Type               *base;          // SAI peripheral base pointer
  sim_clock_gate_name_t   clock_gate;    // SAI peripheral clock gate
  IRQn_Type               tx_irq_num;    // SAI IRQ Number
  IRQn_Type               rx_irq_num;    // SAI Error IRQ Number
#if ((RTE_SAI0) && (RTE_SAI0_DMA_TX == 1))
  SAI_DMA                *tx_dma;        // Pointer to TX DMA structure
#endif
#if ((RTE_SAI0) && (RTE_SAI0_DMA_RX == 1))
  SAI_DMA                *rx_dma;        // Pointer to RX DMA structure
#endif
  uint8_t                 tx_fifo_watermark;
  uint8_t                 rx_fifo_watermark;
  SAI_INFO               *info;          // Run-Time Information
} const SAI_RESOURCES;

#endif /* __SAI_MK64F_H */
