/* -----------------------------------------------------------------------------
 * Copyright (c) 2013-2015 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        16. June 2015
 * $Revision:    V1.2
 *
 * Project:      SPI Driver definitions for Freescale MK64F
 * -------------------------------------------------------------------------- */

#ifndef __SPI_MK64F_H
#define __SPI_MK64F_H

#include <stdint.h>
#include <stdbool.h>
#include <string.h>

#include "Driver_SPI.h"
#include "fsl_device_registers.h"

#include "fsl_clock_manager.h"

#include "fsl_port_hal.h"
#include "fsl_gpio_hal.h"
#include "fsl_dspi_hal.h"

#include "RTE_Components.h"
#include "RTE_Device.h"

#if ((defined(RTE_Drivers_SPI0) || \
      defined(RTE_Drivers_SPI1) || \
      defined(RTE_Drivers_SPI2))   \
     && (RTE_SPI0 == 0)            \
     && (RTE_SPI1 == 0)            \
     && (RTE_SPI2 == 0))
  #error "SPI not configured in RTE_Device.h!"
#endif

/* Driver status flags */
#define SPI_INIT            (uint8_t)(1U << 0)
#define SPI_POWER           (uint8_t)(1U << 1)
#define SPI_SETUP           (uint8_t)(1U << 2)
#define SPI_MBYTE           (uint8_t)(1U << 3)

/* SPI status */
typedef struct _SPI_STATUS {
  uint8_t busy;                         // Transmitter/Receiver busy flag
  uint8_t data_lost;                    // Data lost: Receive overflow / Transmit underflow (cleared on start of transfer operation)
  uint8_t mode_fault;                   // Mode fault detected; optional (cleared on start of transfer operation)
} SPI_STATUS;

// SPI Transfer Information (Run-Time)
typedef struct spi_transfer_info {
  uint32_t                num;          // Total number of frames to transfer
  uint8_t                *rx_buf;       // Received frame buffer
  uint32_t                rx_cnt;       // Current frames received counter
  uint8_t                *tx_buf;       // Transmit frame buffer
  uint32_t                tx_cnt;       // Current frames sent counter
  uint32_t                tx_cmd;       // Transmit command bits
  uint16_t                def_val;      // Default frame value
} SPI_TRANSFER_INFO;

/* SPI Control Information (Run-Time) */
typedef struct spi_ctrl {
  ARM_SPI_SignalEvent_t   cb_event;     // Event Callback
  SPI_STATUS              status;       // Status flags
  SPI_TRANSFER_INFO       xfer;         // Transfer information
  uint8_t                 init;         // Init counter
  uint8_t                 flags;        // Current state flags
} SPI_CTRL;


/* SPI Resources definition */
typedef struct spi_resources {
  SPI_Type               *base;         // Peripheral base pointer
  SPI_CTRL               *ctrl;         // Run-Time information
  IRQn_Type               irq_num;      // Interrupt vector number
  sim_clock_gate_name_t   clock_gate;   // Peripheral clock gate
  dspi_which_pcs_config_t pcs;          // Peripheral chip select number
} const SPI_RESOURCES;

#endif /* __SPI_MK64F_H */
