/* -----------------------------------------------------------------------------
 * Copyright (c) 2013-2015 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        17. June 2015
 * $Revision:    V1.2
 *
 * Project:      USART Driver Definitions for Freescale MK64F
 * -------------------------------------------------------------------------- */

#ifndef __UART_MK64F_H
#define __UART_MK64F_H

#include "Driver_USART.h"
#include "RTE_Device.h"

#include "fsl_device_registers.h"

#include "fsl_clock_manager.h"

#include "fsl_uart_hal.h"

#include "fsl_edma_driver.h"

// USART flags
#define USART_FLAG_INITIALIZED       (1 << 0)
#define USART_FLAG_POWERED           (1 << 1)
#define USART_FLAG_CONFIGURED        (1 << 2)


#define UART_FIFO_SZ_1_DATAWORD      (0U)
#define UART_FIFO_SZ_4_DATAWORD      (1U)
#define UART_FIFO_SZ_8_DATAWORD      (2U)
#define UART_FIFO_SZ_16_DATAWORD     (3U)
#define UART_FIFO_SZ_32_DATAWORD     (4U)
#define UART_FIFO_SZ_64_DATAWORD     (5U)
#define UART_FIFO_SZ_128_DATAWORD    (6U)


// USART Transfer Information (Run-Time)
typedef struct _USART_TRANSFER_INFO {
  uint32_t                rx_num;        // Total number of data to be received
  uint32_t                tx_num;        // Total number of data to be send
  uint8_t                *rx_buf;        // Pointer to in data buffer
  uint8_t                *tx_buf;        // Pointer to out data buffer
  uint32_t                rx_cnt;        // Number of data received
  uint32_t                tx_cnt;        // Number of data sent
} UART_TRANSFER_INFO;

typedef struct _USART_RX_STATUS {
  uint8_t rx_busy;                       // Receiver busy flag
  uint8_t rx_overflow;                   // Receive data overflow detected (cleared on start of next receive operation)
  uint8_t rx_break;                      // Break detected on receive (cleared on start of next receive operation)
  uint8_t rx_framing_error;              // Framing error detected on receive (cleared on start of next receive operation)
  uint8_t rx_parity_error;               // Parity error detected on receive (cleared on start of next receive operation)
} USART_RX_STATUS;

// USART Information (Run-Time)
typedef struct _USART_INFO {
  ARM_USART_SignalEvent_t cb_event;      // Event callback
  USART_RX_STATUS         rx_status;     // Receive Status flags
  UART_TRANSFER_INFO      xfer;          // Transfer information
  uint8_t                 mode;          // USART mode
  uint8_t                 flags;         // USART driver flags
  uint8_t                 rx_fifo_sz;    // Receive FIFO size
  uint8_t                 tx_fifo_sz;    // Transmit FIFO size
  uint32_t                baudrate;      // Baudrate
  uint8_t                 data_bits;     // Data bits
  uint8_t                 send_active;   // Send active flag;
} UART_INFO;

// USART DMA
typedef struct _UART_DMA {
  dma_request_source_t    req_src;       // DMA request source
  edma_chn_state_t       *ch_info;       // Channel info
  edma_transfer_config_t *cfg;           // Transfer configuration structure
} UART_DMA;

// USART Resources definitions
typedef struct {
  const ARM_USART_CAPABILITIES *capabilities;  // Pointer to UART capabilities
        UART_Type              *base;          // UART peripheral base pointer
        sim_clock_gate_name_t   clock_gate;    // UART peripheral clock gate
        IRQn_Type               irq_num;       // UART IRQ Number
        IRQn_Type               err_irq_num;   // UART Error IRQ Number
        UART_DMA               *tx_dma;        // Pointer to TX DMA structure
        UART_DMA               *rx_dma;        // Pointer to TX DMA structure
        uint8_t                 tx_fifo_watermark;
        uint8_t                 rx_fifo_watermark;
        UART_INFO              *info;          // Run-Time Information
} const UART_RESOURCES;

#endif /* __UART_MK64F_H */
