/*----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: LED Flasher for FRDM-K64F
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2004-2015 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include "cmsis_os.h"
#include "board.h"
#include "Board_LED.h"
#include "Board_Buttons.h"

static const uint8_t LED_Table[] = {
  0x01, /* Green=On,  Red=Off, Blue=Off */
  0x02, /* Green=Off, Red=On,  Blue=Off */
  0x04, /* Green=Off, Red=Off, Blue=On  */
  0x03, /* Green=On,  Red=On,  Blue=Off */
  0x05, /* Green=On,  Red=Off, Blue=On  */
  0x06, /* Green=Off, Red=On,  Blue=On  */
  0x07, /* Green=On,  Red=On,  Blue=On  */
  0x00  /* Green=Off, Red=Off, Blue=Off */
};

/*----------------------------------------------------------------------------
  Main function
 *----------------------------------------------------------------------------*/
int main (void) {
  int32_t i;

  hardware_init();                      /* Initialize hardware                */

  LED_Initialize();                     /* LED Initialization                 */
  Buttons_Initialize();                 /* Buttons Initialization             */

  while (1) {                           /* Loop forever                       */
    for (i = 0; i < sizeof(LED_Table); i++) {
      LED_SetOut(LED_Table[i]);         /* Set LED Outputs                    */
      osDelay(1000);                    /* Wait 1s                            */
      while (Buttons_GetState() & 1);   /* Wait while holding INT1 button     */
    }
  }
}
