/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network
 * Copyright (c) 2004-2014 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    HTTP_Server.c
 * Purpose: HTTP Server example
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include "cmsis_os.h"                   /* CMSIS RTOS definitions             */
#include "rl_net.h"                     /* Network definitions                */

#include "Board_LED.h"
#include "Board_Buttons.h"

#include "board.h"                      /* BSP definitions                    */

bool LEDrun;
bool MSGupdate;
char msg_text[64];

static void BlinkLed (void const *arg);
static void Message  (void const *arg);

osThreadDef(BlinkLed, osPriorityNormal, 1, 0);
osThreadDef(Message,  osPriorityNormal, 1, 0);


/// Read analog inputs
uint16_t AD_in (uint32_t ch) {
  /* User analog input is not available */
  return (0);
}

/// Read digital inputs
uint8_t get_button (void) {
  return (Buttons_GetState ());
}

/// IP address change notification
void dhcp_client_notify (uint32_t if_num,
                         dhcpClientOption opt, const uint8_t *val, uint32_t len) {
  if (opt == dhcpClientIPaddress) {
    // IP address has changed
    sprintf (msg_text,"IP address: %s\n", ip4_ntoa (val));
    MSGupdate = true;
  }
}

/*----------------------------------------------------------------------------
  Thread 'Message': printf message handler
 *---------------------------------------------------------------------------*/
static void Message (void const *arg) {

  printf ("       MDK-MW       \n");
  printf (" HTTP Server example\n\n");

  printf ("Waiting for DHCP ...\n");
  fflush (stdout);
  MSGupdate = false;

  while(1) {
    if (MSGupdate == true) {
      printf ("%s\n", msg_text);
      MSGupdate = false;
    }
    osDelay (250);
  }
}

/*----------------------------------------------------------------------------
  Thread 'BlinkLed': Blink the LEDs on an eval board
 *---------------------------------------------------------------------------*/
static void BlinkLed (void const *arg) {
  const uint8_t led_val[] = { 1, 2, 3 };
  int cnt = 0;

  LEDrun = true;
  while(1) {
    // Every 100 ms
    if (LEDrun == true) {
      LED_SetOut (led_val[cnt]);
      if (++cnt >= sizeof(led_val)) {
        cnt = 0;
      }
    }
    osDelay (100);
  }
}

/*----------------------------------------------------------------------------
  Main Thread 'main': Run Network
 *---------------------------------------------------------------------------*/
int main (void) {

  hardware_init();

  LED_Initialize     ();
  Buttons_Initialize ();
  net_initialize     ();

  osThreadCreate (osThread(BlinkLed), NULL);
  osThreadCreate (osThread(Message),  NULL);

  while(1) {
    net_main ();
    osThreadYield ();
  }
}
