/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network
 * Copyright (c) 2004-2014 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    SNMP_Agent.c
 * Purpose: SNMP Agent example
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include "cmsis_os.h"                   /* CMSIS RTOS definitions            */
#include "rl_net.h"                     /* Network definitions               */

#include "Board_LED.h"
#include "Board_Buttons.h"

#include "board.h"                      /* BSP definitions                    */

bool MSGupdate;
char msg_text[64];

static void Message (void const *arg);

osThreadDef(Message, osPriorityNormal, 1, 0);

/// Read digital inputs
uint8_t get_button (void) {
  return (Buttons_GetState ());
}

/// IP address change notification
void dhcp_client_notify (uint32_t if_num,
                         dhcpClientOption opt, const uint8_t *val, uint32_t len) {
  if (opt == dhcpClientIPaddress) {
    // IP address has changed
    sprintf (msg_text,"IP address: %s\n", ip4_ntoa (val));
    MSGupdate = true;
  }
}

/*----------------------------------------------------------------------------
  Thread 'Message': printf message handler
 *---------------------------------------------------------------------------*/
static void Message (void const *arg) {

  printf ("       MDK-MW       \n");
  printf (" SNMP Agent example \n\n");

  printf ("Waiting for DHCP ...\n");
  fflush (stdout);
  MSGupdate = false;

  while(1) {
    if (MSGupdate == true) {
      printf ("%s\n", msg_text);
      MSGupdate = false;
    }
    osDelay (250);
  }
}

/*----------------------------------------------------------------------------
  Main Thread 'main': Run Network
 *---------------------------------------------------------------------------*/
int main (void) {

  hardware_init();

  LED_Initialize     ();
  Buttons_Initialize ();
  net_initialize     ();

  osThreadCreate (osThread(Message), NULL);

  while (1) {
    net_main ();
    osThreadYield ();
  }
}
