/*
 * Copyright (c) 2014, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(__FSL_DMA_REQUEST_H__)
#define __FSL_DMA_REQUEST_H__

/*!
 * @addtogroup edma_request
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/

/*!
 * @brief Structure for the DMA hardware request
 *
 * Defines the structure for the DMA hardware request collections. The user can configure the
 * hardware request into DMAMUX to trigger the DMA transfer accordingly. The index
 * of the hardware request varies according  to the to SoC.
 */
typedef enum _dma_request_source {
#if defined(CPU_MKL14Z32VFM4) || defined(CPU_MKL14Z64VFM4) || defined(CPU_MKL14Z32VFT4) || defined(CPU_MKL14Z64VFT4) || \
    defined(CPU_MKL14Z32VLH4) || defined(CPU_MKL14Z64VLH4) || defined(CPU_MKL14Z32VLK4) || defined(CPU_MKL14Z64VLK4) || \
    defined(CPU_MKL24Z32VFM4) || defined(CPU_MKL24Z64VFM4) || defined(CPU_MKL24Z32VFT4) || defined(CPU_MKL24Z64VFT4) || \
    defined(CPU_MKL24Z32VLH4) || defined(CPU_MKL24Z64VLH4) || defined(CPU_MKL24Z32VLK4) || defined(CPU_MKL24Z64VLK4)
    kDmaRequestMux0Disable = 0|0x100U, 
    kDmaRequestMux0Reserved1 = 1|0x100U, 
    kDmaRequestMux0UART0Rx = 2|0x100U, 
    kDmaRequestMux0LPSCI0Rx = 2|0x100U, 
    kDmaRequestMux0UART0Tx = 3|0x100U, 
    kDmaRequestMux0LPSCI0Tx = 3|0x100U, 
    kDmaRequestMux0UART1Rx = 4|0x100U, 
    kDmaRequestMux0UART1Tx = 5|0x100U, 
    kDmaRequestMux0UART2Rx = 6|0x100U, 
    kDmaRequestMux0UART2Tx = 7|0x100U, 
    kDmaRequestMux0Reserved8 = 8|0x100U, 
    kDmaRequestMux0Reserved9 = 9|0x100U, 
    kDmaRequestMux0Reserved10 = 10|0x100U, 
    kDmaRequestMux0Reserved11 = 11|0x100U, 
    kDmaRequestMux0Reserved12 = 12|0x100U, 
    kDmaRequestMux0Reserved13 = 13|0x100U, 
    kDmaRequestMux0Reserved14 = 14|0x100U, 
    kDmaRequestMux0Reserved15 = 15|0x100U, 
    kDmaRequestMux0SPI0Rx = 16|0x100U, 
    kDmaRequestMux0SPI0Tx = 17|0x100U, 
    kDmaRequestMux0SPI1Rx = 18|0x100U, 
    kDmaRequestMux0SPI1Tx = 19|0x100U, 
    kDmaRequestMux0Reserved20 = 20|0x100U, 
    kDmaRequestMux0Reserved21 = 21|0x100U, 
    kDmaRequestMux0I2C0 = 22|0x100U, 
    kDmaRequestMux0I2C1 = 23|0x100U, 
    kDmaRequestMux0TPM0Channel0 = 24|0x100U, 
    kDmaRequestMux0TPM0Channel1 = 25|0x100U, 
    kDmaRequestMux0TPM0Channel2 = 26|0x100U, 
    kDmaRequestMux0TPM0Channel3 = 27|0x100U, 
    kDmaRequestMux0TPM0Channel4 = 28|0x100U, 
    kDmaRequestMux0TPM0Channel5 = 29|0x100U, 
    kDmaRequestMux0Reserved30 = 30|0x100U, 
    kDmaRequestMux0Reserved31 = 31|0x100U, 
    kDmaRequestMux0TPM1Channel0 = 32|0x100U, 
    kDmaRequestMux0TPM1Channel1 = 33|0x100U, 
    kDmaRequestMux0TPM2Channel0 = 34|0x100U, 
    kDmaRequestMux0TPM2Channel1 = 35|0x100U, 
    kDmaRequestMux0Reserved36 = 36|0x100U, 
    kDmaRequestMux0Reserved37 = 37|0x100U, 
    kDmaRequestMux0Reserved38 = 38|0x100U, 
    kDmaRequestMux0Reserved39 = 39|0x100U, 
    kDmaRequestMux0ADC0 = 40|0x100U, 
    kDmaRequestMux0Reserved41 = 41|0x100U, 
    kDmaRequestMux0CMP0 = 42|0x100U, 
    kDmaRequestMux0Reserved43 = 43|0x100U, 
    kDmaRequestMux0Reserved44 = 44|0x100U, 
    kDmaRequestMux0Reserved45 = 45|0x100U, 
    kDmaRequestMux0Reserved46 = 46|0x100U, 
    kDmaRequestMux0Reserved47 = 47|0x100U, 
    kDmaRequestMux0Reserved48 = 48|0x100U, 
    kDmaRequestMux0PortA = 49|0x100U, 
    kDmaRequestMux0Reserved50 = 50|0x100U, 
    kDmaRequestMux0Reserved51 = 51|0x100U, 
    kDmaRequestMux0PortD = 52|0x100U, 
    kDmaRequestMux0Reserved53 = 53|0x100U, 
    kDmaRequestMux0TPM0Overflow = 54|0x100U, 
    kDmaRequestMux0TPM1Overflow = 55|0x100U, 
    kDmaRequestMux0TPM2Overflow = 56|0x100U, 
    kDmaRequestMux0Reserved57 = 57|0x100U, 
    kDmaRequestMux0Reserved58 = 58|0x100U, 
    kDmaRequestMux0Reserved59 = 59|0x100U, 
    kDmaRequestMux0AlwaysOn60 = 60|0x100U, 
    kDmaRequestMux0AlwaysOn61 = 61|0x100U, 
    kDmaRequestMux0AlwaysOn62 = 62|0x100U, 
    kDmaRequestMux0AlwaysOn63 = 63|0x100U, 
#elif defined(CPU_MKL15Z128CAD4) || defined(CPU_MKL15Z32VFM4) || defined(CPU_MKL15Z64VFM4) || defined(CPU_MKL15Z128VFM4) || \
    defined(CPU_MKL15Z32VFT4) || defined(CPU_MKL15Z64VFT4) || defined(CPU_MKL15Z128VFT4) || defined(CPU_MKL15Z32VLH4) || \
    defined(CPU_MKL15Z64VLH4) || defined(CPU_MKL15Z128VLH4) || defined(CPU_MKL15Z32VLK4) || defined(CPU_MKL15Z64VLK4) || \
    defined(CPU_MKL15Z128VLK4) || defined(CPU_MKL25Z32VFM4) || defined(CPU_MKL25Z64VFM4) || defined(CPU_MKL25Z128VFM4) || \
    defined(CPU_MKL25Z32VFT4) || defined(CPU_MKL25Z64VFT4) || defined(CPU_MKL25Z128VFT4) || defined(CPU_MKL25Z32VLH4) || \
    defined(CPU_MKL25Z64VLH4) || defined(CPU_MKL25Z128VLH4) || defined(CPU_MKL25Z32VLK4) || defined(CPU_MKL25Z64VLK4) || \
    defined(CPU_MKL25Z128VLK4)
    kDmaRequestMux0Disable = 0|0x100U, 
    kDmaRequestMux0Reserved1 = 1|0x100U, 
    kDmaRequestMux0UART0Rx = 2|0x100U, 
    kDmaRequestMux0LPSCI0Rx = 2|0x100U, 
    kDmaRequestMux0UART0Tx = 3|0x100U, 
    kDmaRequestMux0LPSCI0Tx = 3|0x100U, 
    kDmaRequestMux0UART1Rx = 4|0x100U, 
    kDmaRequestMux0UART1Tx = 5|0x100U, 
    kDmaRequestMux0UART2Rx = 6|0x100U, 
    kDmaRequestMux0UART2Tx = 7|0x100U, 
    kDmaRequestMux0Reserved8 = 8|0x100U, 
    kDmaRequestMux0Reserved9 = 9|0x100U, 
    kDmaRequestMux0Reserved10 = 10|0x100U, 
    kDmaRequestMux0Reserved11 = 11|0x100U, 
    kDmaRequestMux0Reserved12 = 12|0x100U, 
    kDmaRequestMux0Reserved13 = 13|0x100U, 
    kDmaRequestMux0Reserved14 = 14|0x100U, 
    kDmaRequestMux0Reserved15 = 15|0x100U, 
    kDmaRequestMux0SPI0Rx = 16|0x100U, 
    kDmaRequestMux0SPI0Tx = 17|0x100U, 
    kDmaRequestMux0SPI1Rx = 18|0x100U, 
    kDmaRequestMux0SPI1Tx = 19|0x100U, 
    kDmaRequestMux0Reserved20 = 20|0x100U, 
    kDmaRequestMux0Reserved21 = 21|0x100U, 
    kDmaRequestMux0I2C0 = 22|0x100U, 
    kDmaRequestMux0I2C1 = 23|0x100U, 
    kDmaRequestMux0TPM0Channel0 = 24|0x100U, 
    kDmaRequestMux0TPM0Channel1 = 25|0x100U, 
    kDmaRequestMux0TPM0Channel2 = 26|0x100U, 
    kDmaRequestMux0TPM0Channel3 = 27|0x100U, 
    kDmaRequestMux0TPM0Channel4 = 28|0x100U, 
    kDmaRequestMux0TPM0Channel5 = 29|0x100U, 
    kDmaRequestMux0Reserved30 = 30|0x100U, 
    kDmaRequestMux0Reserved31 = 31|0x100U, 
    kDmaRequestMux0TPM1Channel0 = 32|0x100U, 
    kDmaRequestMux0TPM1Channel1 = 33|0x100U, 
    kDmaRequestMux0TPM2Channel0 = 34|0x100U, 
    kDmaRequestMux0TPM2Channel1 = 35|0x100U, 
    kDmaRequestMux0Reserved36 = 36|0x100U, 
    kDmaRequestMux0Reserved37 = 37|0x100U, 
    kDmaRequestMux0Reserved38 = 38|0x100U, 
    kDmaRequestMux0Reserved39 = 39|0x100U, 
    kDmaRequestMux0ADC0 = 40|0x100U, 
    kDmaRequestMux0Reserved41 = 41|0x100U, 
    kDmaRequestMux0CMP0 = 42|0x100U, 
    kDmaRequestMux0Reserved43 = 43|0x100U, 
    kDmaRequestMux0Reserved44 = 44|0x100U, 
    kDmaRequestMux0DAC0 = 45|0x100U, 
    kDmaRequestMux0Reserved46 = 46|0x100U, 
    kDmaRequestMux0Reserved47 = 47|0x100U, 
    kDmaRequestMux0Reserved48 = 48|0x100U, 
    kDmaRequestMux0PortA = 49|0x100U, 
    kDmaRequestMux0Reserved50 = 50|0x100U, 
    kDmaRequestMux0Reserved51 = 51|0x100U, 
    kDmaRequestMux0PortD = 52|0x100U, 
    kDmaRequestMux0Reserved53 = 53|0x100U, 
    kDmaRequestMux0TPM0Overflow = 54|0x100U, 
    kDmaRequestMux0TPM1Overflow = 55|0x100U, 
    kDmaRequestMux0TPM2Overflow = 56|0x100U, 
    kDmaRequestMux0TSI0 = 57|0x100U, 
    kDmaRequestMux0Reserved58 = 58|0x100U, 
    kDmaRequestMux0Reserved59 = 59|0x100U, 
    kDmaRequestMux0AlwaysOn60 = 60|0x100U, 
    kDmaRequestMux0AlwaysOn61 = 61|0x100U, 
    kDmaRequestMux0AlwaysOn62 = 62|0x100U, 
    kDmaRequestMux0AlwaysOn63 = 63|0x100U, 
#elif defined(CPU_MKL16Z32VFM4) || defined(CPU_MKL16Z64VFM4) || defined(CPU_MKL16Z128VFM4) || defined(CPU_MKL16Z32VFT4) || \
    defined(CPU_MKL16Z64VFT4) || defined(CPU_MKL16Z128VFT4) || defined(CPU_MKL16Z32VLH4) || defined(CPU_MKL16Z64VLH4) || \
    defined(CPU_MKL16Z128VLH4) || defined(CPU_MKL26Z128CAL4) || defined(CPU_MKL26Z32VFM4) || defined(CPU_MKL26Z64VFM4) || \
    defined(CPU_MKL26Z128VFM4) || defined(CPU_MKL26Z32VFT4) || defined(CPU_MKL26Z64VFT4) || defined(CPU_MKL26Z128VFT4) || \
    defined(CPU_MKL26Z32VLH4) || defined(CPU_MKL26Z64VLH4) || defined(CPU_MKL26Z128VLH4)
    kDmaRequestMux0Disable = 0|0x100U, 
    kDmaRequestMux0Reserved1 = 1|0x100U, 
    kDmaRequestMux0UART0Rx = 2|0x100U, 
    kDmaRequestMux0LPSCI0Rx = 2|0x100U, 
    kDmaRequestMux0UART0Tx = 3|0x100U, 
    kDmaRequestMux0LPSCI0Tx = 3|0x100U, 
    kDmaRequestMux0UART1Rx = 4|0x100U, 
    kDmaRequestMux0UART1Tx = 5|0x100U, 
    kDmaRequestMux0UART2Rx = 6|0x100U, 
    kDmaRequestMux0UART2Tx = 7|0x100U, 
    kDmaRequestMux0Reserved8 = 8|0x100U, 
    kDmaRequestMux0Reserved9 = 9|0x100U, 
    kDmaRequestMux0Reserved10 = 10|0x100U, 
    kDmaRequestMux0Reserved11 = 11|0x100U, 
    kDmaRequestMux0Reserved12 = 12|0x100U, 
    kDmaRequestMux0Reserved13 = 13|0x100U, 
    kDmaRequestMux0I2S0Rx = 14|0x100U, 
    kDmaRequestMux0I2S0Tx = 15|0x100U, 
    kDmaRequestMux0SPI0Rx = 16|0x100U, 
    kDmaRequestMux0SPI0Tx = 17|0x100U, 
    kDmaRequestMux0SPI1Rx = 18|0x100U, 
    kDmaRequestMux0SPI1Tx = 19|0x100U, 
    kDmaRequestMux0Reserved20 = 20|0x100U, 
    kDmaRequestMux0Reserved21 = 21|0x100U, 
    kDmaRequestMux0I2C0 = 22|0x100U, 
    kDmaRequestMux0I2C1 = 23|0x100U, 
    kDmaRequestMux0TPM0Channel0 = 24|0x100U, 
    kDmaRequestMux0TPM0Channel1 = 25|0x100U, 
    kDmaRequestMux0TPM0Channel2 = 26|0x100U, 
    kDmaRequestMux0TPM0Channel3 = 27|0x100U, 
    kDmaRequestMux0TPM0Channel4 = 28|0x100U, 
    kDmaRequestMux0TPM0Channel5 = 29|0x100U, 
    kDmaRequestMux0Reserved30 = 30|0x100U, 
    kDmaRequestMux0Reserved31 = 31|0x100U, 
    kDmaRequestMux0TPM1Channel0 = 32|0x100U, 
    kDmaRequestMux0TPM1Channel1 = 33|0x100U, 
    kDmaRequestMux0TPM2Channel0 = 34|0x100U, 
    kDmaRequestMux0TPM2Channel1 = 35|0x100U, 
    kDmaRequestMux0Reserved36 = 36|0x100U, 
    kDmaRequestMux0Reserved37 = 37|0x100U, 
    kDmaRequestMux0Reserved38 = 38|0x100U, 
    kDmaRequestMux0Reserved39 = 39|0x100U, 
    kDmaRequestMux0ADC0 = 40|0x100U, 
    kDmaRequestMux0Reserved41 = 41|0x100U, 
    kDmaRequestMux0CMP0 = 42|0x100U, 
    kDmaRequestMux0Reserved43 = 43|0x100U, 
    kDmaRequestMux0Reserved44 = 44|0x100U, 
    kDmaRequestMux0DAC0 = 45|0x100U, 
    kDmaRequestMux0Reserved46 = 46|0x100U, 
    kDmaRequestMux0Reserved47 = 47|0x100U, 
    kDmaRequestMux0Reserved48 = 48|0x100U, 
    kDmaRequestMux0PortA = 49|0x100U, 
    kDmaRequestMux0Reserved50 = 50|0x100U, 
    kDmaRequestMux0PortC = 51|0x100U, 
    kDmaRequestMux0PortD = 52|0x100U, 
    kDmaRequestMux0Reserved53 = 53|0x100U, 
    kDmaRequestMux0TPM0Overflow = 54|0x100U, 
    kDmaRequestMux0TPM1Overflow = 55|0x100U, 
    kDmaRequestMux0TPM2Overflow = 56|0x100U, 
    kDmaRequestMux0Reserved57 = 57|0x100U, 
    kDmaRequestMux0Reserved58 = 58|0x100U, 
    kDmaRequestMux0Reserved59 = 59|0x100U, 
    kDmaRequestMux0AlwaysOn60 = 60|0x100U, 
    kDmaRequestMux0AlwaysOn61 = 61|0x100U, 
    kDmaRequestMux0AlwaysOn62 = 62|0x100U, 
    kDmaRequestMux0AlwaysOn63 = 63|0x100U, 
#elif defined(CPU_MKL16Z256VLH4) || defined(CPU_MKL16Z256VMP4) || defined(CPU_MKL26Z256VLH4) || defined(CPU_MKL26Z128VLL4) || \
    defined(CPU_MKL26Z256VLL4) || defined(CPU_MKL26Z128VMC4) || defined(CPU_MKL26Z256VMC4) || defined(CPU_MKL26Z256VMP4) || \
    defined(CPU_MKL36Z64VLH4) || defined(CPU_MKL36Z128VLH4) || defined(CPU_MKL36Z256VLH4) || defined(CPU_MKL36Z64VLL4) || \
    defined(CPU_MKL36Z128VLL4) || defined(CPU_MKL36Z256VLL4) || defined(CPU_MKL36Z128VMC4) || defined(CPU_MKL36Z256VMC4) || \
    defined(CPU_MKL36Z256VMP4) || defined(CPU_MKL46Z128VLH4) || defined(CPU_MKL46Z256VLH4) || defined(CPU_MKL46Z128VLL4) || \
    defined(CPU_MKL46Z256VLL4) || defined(CPU_MKL46Z128VMC4) || defined(CPU_MKL46Z256VMC4) || defined(CPU_MKL46Z256VMP4)
    kDmaRequestMux0Disable = 0|0x100U, 
    kDmaRequestMux0Reserved1 = 1|0x100U, 
    kDmaRequestMux0UART0Rx = 2|0x100U, 
    kDmaRequestMux0LPSCI0Rx = 2|0x100U, 
    kDmaRequestMux0UART0Tx = 3|0x100U, 
    kDmaRequestMux0LPSCI0Tx = 3|0x100U, 
    kDmaRequestMux0UART1Rx = 4|0x100U, 
    kDmaRequestMux0UART1Tx = 5|0x100U, 
    kDmaRequestMux0UART2Rx = 6|0x100U, 
    kDmaRequestMux0UART2Tx = 7|0x100U, 
    kDmaRequestMux0Reserved8 = 8|0x100U, 
    kDmaRequestMux0Reserved9 = 9|0x100U, 
    kDmaRequestMux0Reserved10 = 10|0x100U, 
    kDmaRequestMux0Reserved11 = 11|0x100U, 
    kDmaRequestMux0Reserved12 = 12|0x100U, 
    kDmaRequestMux0Reserved13 = 13|0x100U, 
    kDmaRequestMux0I2S0Rx = 14|0x100U, 
    kDmaRequestMux0I2S0Tx = 15|0x100U, 
    kDmaRequestMux0SPI0Rx = 16|0x100U, 
    kDmaRequestMux0SPI0Tx = 17|0x100U, 
    kDmaRequestMux0SPI1Rx = 18|0x100U, 
    kDmaRequestMux0SPI1Tx = 19|0x100U, 
    kDmaRequestMux0Reserved20 = 20|0x100U, 
    kDmaRequestMux0Reserved21 = 21|0x100U, 
    kDmaRequestMux0I2C0 = 22|0x100U, 
    kDmaRequestMux0I2C1 = 23|0x100U, 
    kDmaRequestMux0TPM0Channel0 = 24|0x100U, 
    kDmaRequestMux0TPM0Channel1 = 25|0x100U, 
    kDmaRequestMux0TPM0Channel2 = 26|0x100U, 
    kDmaRequestMux0TPM0Channel3 = 27|0x100U, 
    kDmaRequestMux0TPM0Channel4 = 28|0x100U, 
    kDmaRequestMux0TPM0Channel5 = 29|0x100U, 
    kDmaRequestMux0Reserved30 = 30|0x100U, 
    kDmaRequestMux0Reserved31 = 31|0x100U, 
    kDmaRequestMux0TPM1Channel0 = 32|0x100U, 
    kDmaRequestMux0TPM1Channel1 = 33|0x100U, 
    kDmaRequestMux0TPM2Channel0 = 34|0x100U, 
    kDmaRequestMux0TPM2Channel1 = 35|0x100U, 
    kDmaRequestMux0Reserved36 = 36|0x100U, 
    kDmaRequestMux0Reserved37 = 37|0x100U, 
    kDmaRequestMux0Reserved38 = 38|0x100U, 
    kDmaRequestMux0Reserved39 = 39|0x100U, 
    kDmaRequestMux0ADC0 = 40|0x100U, 
    kDmaRequestMux0Reserved41 = 41|0x100U, 
    kDmaRequestMux0CMP0 = 42|0x100U, 
    kDmaRequestMux0Reserved43 = 43|0x100U, 
    kDmaRequestMux0Reserved44 = 44|0x100U, 
    kDmaRequestMux0DAC0 = 45|0x100U, 
    kDmaRequestMux0Reserved46 = 46|0x100U, 
    kDmaRequestMux0Reserved47 = 47|0x100U, 
    kDmaRequestMux0Reserved48 = 48|0x100U, 
    kDmaRequestMux0PortA = 49|0x100U, 
    kDmaRequestMux0Reserved50 = 50|0x100U, 
    kDmaRequestMux0PortC = 51|0x100U, 
    kDmaRequestMux0PortD = 52|0x100U, 
    kDmaRequestMux0Reserved53 = 53|0x100U, 
    kDmaRequestMux0TPM0Overflow = 54|0x100U, 
    kDmaRequestMux0TPM1Overflow = 55|0x100U, 
    kDmaRequestMux0TPM2Overflow = 56|0x100U, 
    kDmaRequestMux0TSI = 57|0x100U, 
    kDmaRequestMux0Reserved58 = 58|0x100U, 
    kDmaRequestMux0Reserved59 = 59|0x100U, 
    kDmaRequestMux0AlwaysOn60 = 60|0x100U, 
    kDmaRequestMux0AlwaysOn61 = 61|0x100U, 
    kDmaRequestMux0AlwaysOn62 = 62|0x100U, 
    kDmaRequestMux0AlwaysOn63 = 63|0x100U, 
#elif defined(CPU_MKL17Z128VFM4) || defined(CPU_MKL17Z256VFM4) || defined(CPU_MKL17Z128VFT4) || defined(CPU_MKL17Z256VFT4) || \
    defined(CPU_MKL17Z128VLH4) || defined(CPU_MKL17Z256VLH4) || defined(CPU_MKL17Z128VMP4) || defined(CPU_MKL17Z256VMP4) || \
    defined(CPU_MKL27Z128VFM4) || defined(CPU_MKL27Z256VFM4) || defined(CPU_MKL27Z128VFT4) || defined(CPU_MKL27Z256VFT4) || \
    defined(CPU_MKL27Z128VLH4) || defined(CPU_MKL27Z256VLH4) || defined(CPU_MKL27Z128VMP4) || defined(CPU_MKL27Z256VMP4) || \
    defined(CPU_MKL33Z128VLH4) || defined(CPU_MKL33Z256VLH4) || defined(CPU_MKL33Z128VMP4) || defined(CPU_MKL33Z256VMP4) || \
    defined(CPU_MKL43Z128VLH4) || defined(CPU_MKL43Z256VLH4) || defined(CPU_MKL43Z128VMP4) || defined(CPU_MKL43Z256VMP4)
    kDmaRequestMux0Disable = 0|0x100U, 
    kDmaRequestMux0Reserved1 = 1|0x100U, 
    kDmaRequestMux0LPUART0Rx = 2|0x100U, 
    kDmaRequestMux0LPUART0Tx = 3|0x100U, 
    kDmaRequestMux0LPUART1Rx = 4|0x100U, 
    kDmaRequestMux0LPUART1Tx = 5|0x100U, 
    kDmaRequestMux0UART2Rx = 6|0x100U, 
    kDmaRequestMux0UART2Tx = 7|0x100U, 
    kDmaRequestMux0Reserved8 = 8|0x100U, 
    kDmaRequestMux0Reserved9 = 9|0x100U, 
    kDmaRequestMux0FlexIOChannel0 = 10|0x100U, 
    kDmaRequestMux0FlexIOChannel1 = 11|0x100U, 
    kDmaRequestMux0FlexIOChannel2 = 12|0x100U, 
    kDmaRequestMux0FlexIOChannel3 = 13|0x100U, 
    kDmaRequestMux0I2S0Rx = 14|0x100U, 
    kDmaRequestMux0I2S0Tx = 15|0x100U, 
    kDmaRequestMux0SPI0Rx = 16|0x100U, 
    kDmaRequestMux0SPI0Tx = 17|0x100U, 
    kDmaRequestMux0SPI1Rx = 18|0x100U, 
    kDmaRequestMux0SPI1Tx = 19|0x100U, 
    kDmaRequestMux0Reserved20 = 20|0x100U, 
    kDmaRequestMux0Reserved21 = 21|0x100U, 
    kDmaRequestMux0I2C0 = 22|0x100U, 
    kDmaRequestMux0I2C1 = 23|0x100U, 
    kDmaRequestMux0TPM0Channel0 = 24|0x100U, 
    kDmaRequestMux0TPM0Channel1 = 25|0x100U, 
    kDmaRequestMux0TPM0Channel2 = 26|0x100U, 
    kDmaRequestMux0TPM0Channel3 = 27|0x100U, 
    kDmaRequestMux0TPM0Channel4 = 28|0x100U, 
    kDmaRequestMux0TPM0Channel5 = 29|0x100U, 
    kDmaRequestMux0Reserved30 = 30|0x100U, 
    kDmaRequestMux0Reserved31 = 31|0x100U, 
    kDmaRequestMux0TPM1Channel0 = 32|0x100U, 
    kDmaRequestMux0TPM1Channel1 = 33|0x100U, 
    kDmaRequestMux0TPM2Channel0 = 34|0x100U, 
    kDmaRequestMux0TPM2Channel1 = 35|0x100U, 
    kDmaRequestMux0Reserved36 = 36|0x100U, 
    kDmaRequestMux0Reserved37 = 37|0x100U, 
    kDmaRequestMux0Reserved38 = 38|0x100U, 
    kDmaRequestMux0Reserved39 = 39|0x100U, 
    kDmaRequestMux0ADC0 = 40|0x100U, 
    kDmaRequestMux0Reserved41 = 41|0x100U, 
    kDmaRequestMux0CMP0 = 42|0x100U, 
    kDmaRequestMux0Reserved43 = 43|0x100U, 
    kDmaRequestMux0Reserved44 = 44|0x100U, 
    kDmaRequestMux0DAC0 = 45|0x100U, 
    kDmaRequestMux0Reserved46 = 46|0x100U, 
    kDmaRequestMux0Reserved47 = 47|0x100U, 
    kDmaRequestMux0Reserved48 = 48|0x100U, 
    kDmaRequestMux0PortA = 49|0x100U, 
    kDmaRequestMux0Reserved50 = 50|0x100U, 
    kDmaRequestMux0PortC = 51|0x100U, 
    kDmaRequestMux0PortD = 52|0x100U, 
    kDmaRequestMux0Reserved53 = 53|0x100U, 
    kDmaRequestMux0TPM0Overflow = 54|0x100U, 
    kDmaRequestMux0TPM1Overflow = 55|0x100U, 
    kDmaRequestMux0TPM2Overflow = 56|0x100U, 
    kDmaRequestMux0Reserved57 = 57|0x100U, 
    kDmaRequestMux0Reserved58 = 58|0x100U, 
    kDmaRequestMux0Reserved59 = 59|0x100U, 
    kDmaRequestMux0AlwaysOn60 = 60|0x100U, 
    kDmaRequestMux0AlwaysOn61 = 61|0x100U, 
    kDmaRequestMux0AlwaysOn62 = 62|0x100U, 
    kDmaRequestMux0AlwaysOn63 = 63|0x100U, 
#elif defined(CPU_MKL17Z32VDA4) || defined(CPU_MKL17Z64VDA4) || defined(CPU_MKL17Z32VFM4) || defined(CPU_MKL17Z64VFM4) || \
    defined(CPU_MKL17Z32VFT4) || defined(CPU_MKL17Z64VFT4) || defined(CPU_MKL17Z32VLH4) || defined(CPU_MKL17Z64VLH4) || \
    defined(CPU_MKL17Z32VMP4) || defined(CPU_MKL17Z64VMP4) || defined(CPU_MKL27Z32VDA4) || defined(CPU_MKL27Z64VDA4) || \
    defined(CPU_MKL27Z32VFM4) || defined(CPU_MKL27Z64VFM4) || defined(CPU_MKL27Z32VFT4) || defined(CPU_MKL27Z64VFT4) || \
    defined(CPU_MKL27Z32VLH4) || defined(CPU_MKL27Z64VLH4) || defined(CPU_MKL27Z32VMP4) || defined(CPU_MKL27Z64VMP4)
    kDmaRequestMux0Disable = 0|0x100U, 
    kDmaRequestMux0Reserved1 = 1|0x100U, 
    kDmaRequestMux0LPUART0Rx = 2|0x100U, 
    kDmaRequestMux0LPUART0Tx = 3|0x100U, 
    kDmaRequestMux0LPUART1Rx = 4|0x100U, 
    kDmaRequestMux0LPUART1Tx = 5|0x100U, 
    kDmaRequestMux0UART2Rx = 6|0x100U, 
    kDmaRequestMux0UART2Tx = 7|0x100U, 
    kDmaRequestMux0Reserved8 = 8|0x100U, 
    kDmaRequestMux0Reserved9 = 9|0x100U, 
    kDmaRequestMux0FlexIOChannel0 = 10|0x100U, 
    kDmaRequestMux0FlexIOChannel1 = 11|0x100U, 
    kDmaRequestMux0FlexIOChannel2 = 12|0x100U, 
    kDmaRequestMux0FlexIOChannel3 = 13|0x100U, 
    kDmaRequestMux0Reserved14 = 14|0x100U, 
    kDmaRequestMux0Reserved15 = 15|0x100U, 
    kDmaRequestMux0SPI0Rx = 16|0x100U, 
    kDmaRequestMux0SPI0Tx = 17|0x100U, 
    kDmaRequestMux0SPI1Rx = 18|0x100U, 
    kDmaRequestMux0SPI1Tx = 19|0x100U, 
    kDmaRequestMux0Reserved20 = 20|0x100U, 
    kDmaRequestMux0Reserved21 = 21|0x100U, 
    kDmaRequestMux0I2C0 = 22|0x100U, 
    kDmaRequestMux0I2C1 = 23|0x100U, 
    kDmaRequestMux0TPM0Channel0 = 24|0x100U, 
    kDmaRequestMux0TPM0Channel1 = 25|0x100U, 
    kDmaRequestMux0TPM0Channel2 = 26|0x100U, 
    kDmaRequestMux0TPM0Channel3 = 27|0x100U, 
    kDmaRequestMux0TPM0Channel4 = 28|0x100U, 
    kDmaRequestMux0TPM0Channel5 = 29|0x100U, 
    kDmaRequestMux0Reserved30 = 30|0x100U, 
    kDmaRequestMux0Reserved31 = 31|0x100U, 
    kDmaRequestMux0TPM1Channel0 = 32|0x100U, 
    kDmaRequestMux0TPM1Channel1 = 33|0x100U, 
    kDmaRequestMux0TPM2Channel0 = 34|0x100U, 
    kDmaRequestMux0TPM2Channel1 = 35|0x100U, 
    kDmaRequestMux0Reserved36 = 36|0x100U, 
    kDmaRequestMux0Reserved37 = 37|0x100U, 
    kDmaRequestMux0Reserved38 = 38|0x100U, 
    kDmaRequestMux0Reserved39 = 39|0x100U, 
    kDmaRequestMux0ADC0 = 40|0x100U, 
    kDmaRequestMux0Reserved41 = 41|0x100U, 
    kDmaRequestMux0CMP0 = 42|0x100U, 
    kDmaRequestMux0Reserved43 = 43|0x100U, 
    kDmaRequestMux0Reserved44 = 44|0x100U, 
    kDmaRequestMux0Reserved45 = 45|0x100U, 
    kDmaRequestMux0Reserved46 = 46|0x100U, 
    kDmaRequestMux0Reserved47 = 47|0x100U, 
    kDmaRequestMux0Reserved48 = 48|0x100U, 
    kDmaRequestMux0PortA = 49|0x100U, 
    kDmaRequestMux0PortB = 50|0x100U, 
    kDmaRequestMux0PortC = 51|0x100U, 
    kDmaRequestMux0PortD = 52|0x100U, 
    kDmaRequestMux0PortE = 53|0x100U, 
    kDmaRequestMux0TPM0Overflow = 54|0x100U, 
    kDmaRequestMux0TPM1Overflow = 55|0x100U, 
    kDmaRequestMux0TPM2Overflow = 56|0x100U, 
    kDmaRequestMux0Reserved57 = 57|0x100U, 
    kDmaRequestMux0Reserved58 = 58|0x100U, 
    kDmaRequestMux0Reserved59 = 59|0x100U, 
    kDmaRequestMux0AlwaysOn60 = 60|0x100U, 
    kDmaRequestMux0AlwaysOn61 = 61|0x100U, 
    kDmaRequestMux0AlwaysOn62 = 62|0x100U, 
    kDmaRequestMux0AlwaysOn63 = 63|0x100U, 
#elif defined(CPU_MKL34Z64VLH4) || defined(CPU_MKL34Z64VLL4)
    kDmaRequestMux0Disable = 0|0x100U, 
    kDmaRequestMux0Reserved1 = 1|0x100U, 
    kDmaRequestMux0UART0Rx = 2|0x100U, 
    kDmaRequestMux0LPSCI0Rx = 2|0x100U, 
    kDmaRequestMux0UART0Tx = 3|0x100U, 
    kDmaRequestMux0LPSCI0Tx = 3|0x100U, 
    kDmaRequestMux0UART1Rx = 4|0x100U, 
    kDmaRequestMux0UART1Tx = 5|0x100U, 
    kDmaRequestMux0UART2Rx = 6|0x100U, 
    kDmaRequestMux0UART2Tx = 7|0x100U, 
    kDmaRequestMux0Reserved8 = 8|0x100U, 
    kDmaRequestMux0Reserved9 = 9|0x100U, 
    kDmaRequestMux0Reserved10 = 10|0x100U, 
    kDmaRequestMux0Reserved11 = 11|0x100U, 
    kDmaRequestMux0Reserved12 = 12|0x100U, 
    kDmaRequestMux0Reserved13 = 13|0x100U, 
    kDmaRequestMux0Reserved14 = 14|0x100U, 
    kDmaRequestMux0Reserved15 = 15|0x100U, 
    kDmaRequestMux0SPI0Rx = 16|0x100U, 
    kDmaRequestMux0SPI0Tx = 17|0x100U, 
    kDmaRequestMux0SPI1Rx = 18|0x100U, 
    kDmaRequestMux0SPI1Tx = 19|0x100U, 
    kDmaRequestMux0Reserved20 = 20|0x100U, 
    kDmaRequestMux0Reserved21 = 21|0x100U, 
    kDmaRequestMux0I2C0 = 22|0x100U, 
    kDmaRequestMux0I2C1 = 23|0x100U, 
    kDmaRequestMux0TPM0Channel0 = 24|0x100U, 
    kDmaRequestMux0TPM0Channel1 = 25|0x100U, 
    kDmaRequestMux0TPM0Channel2 = 26|0x100U, 
    kDmaRequestMux0TPM0Channel3 = 27|0x100U, 
    kDmaRequestMux0TPM0Channel4 = 28|0x100U, 
    kDmaRequestMux0TPM0Channel5 = 29|0x100U, 
    kDmaRequestMux0Reserved30 = 30|0x100U, 
    kDmaRequestMux0Reserved31 = 31|0x100U, 
    kDmaRequestMux0TPM1Channel0 = 32|0x100U, 
    kDmaRequestMux0TPM1Channel1 = 33|0x100U, 
    kDmaRequestMux0TPM2Channel0 = 34|0x100U, 
    kDmaRequestMux0TPM2Channel1 = 35|0x100U, 
    kDmaRequestMux0Reserved36 = 36|0x100U, 
    kDmaRequestMux0Reserved37 = 37|0x100U, 
    kDmaRequestMux0Reserved38 = 38|0x100U, 
    kDmaRequestMux0Reserved39 = 39|0x100U, 
    kDmaRequestMux0ADC0 = 40|0x100U, 
    kDmaRequestMux0Reserved41 = 41|0x100U, 
    kDmaRequestMux0CMP0 = 42|0x100U, 
    kDmaRequestMux0Reserved43 = 43|0x100U, 
    kDmaRequestMux0Reserved44 = 44|0x100U, 
    kDmaRequestMux0Reserved45 = 45|0x100U, 
    kDmaRequestMux0Reserved46 = 46|0x100U, 
    kDmaRequestMux0Reserved47 = 47|0x100U, 
    kDmaRequestMux0Reserved48 = 48|0x100U, 
    kDmaRequestMux0PortA = 49|0x100U, 
    kDmaRequestMux0Reserved50 = 50|0x100U, 
    kDmaRequestMux0PortC = 51|0x100U, 
    kDmaRequestMux0PortD = 52|0x100U, 
    kDmaRequestMux0Reserved53 = 53|0x100U, 
    kDmaRequestMux0TPM0Overflow = 54|0x100U, 
    kDmaRequestMux0TPM1Overflow = 55|0x100U, 
    kDmaRequestMux0TPM2Overflow = 56|0x100U, 
    kDmaRequestMux0Reserved57 = 57|0x100U, 
    kDmaRequestMux0Reserved58 = 58|0x100U, 
    kDmaRequestMux0Reserved59 = 59|0x100U, 
    kDmaRequestMux0AlwaysOn60 = 60|0x100U, 
    kDmaRequestMux0AlwaysOn61 = 61|0x100U, 
    kDmaRequestMux0AlwaysOn62 = 62|0x100U, 
    kDmaRequestMux0AlwaysOn63 = 63|0x100U, 
#elif defined(CPU_MKW01Z128CHN4)
    kDmaRequestMux0Disable = 0|0x100U, 
    kDmaRequestMux0Reserved1 = 1|0x100U, 
    kDmaRequestMux0UART0Rx = 2|0x100U, 
    kDmaRequestMux0LPSCI0Rx = 2|0x100U, 
    kDmaRequestMux0UART0Tx = 3|0x100U, 
    kDmaRequestMux0LPSCI0Tx = 3|0x100U, 
    kDmaRequestMux0UART1Rx = 4|0x100U, 
    kDmaRequestMux0UART1Tx = 5|0x100U, 
    kDmaRequestMux0UART2Rx = 6|0x100U, 
    kDmaRequestMux0UART2Tx = 7|0x100U, 
    kDmaRequestMux0Reserved8 = 8|0x100U, 
    kDmaRequestMux0Reserved9 = 9|0x100U, 
    kDmaRequestMux0Reserved10 = 10|0x100U, 
    kDmaRequestMux0Reserved11 = 11|0x100U, 
    kDmaRequestMux0Reserved12 = 12|0x100U, 
    kDmaRequestMux0Reserved13 = 13|0x100U, 
    kDmaRequestMux0Reserved14 = 14|0x100U, 
    kDmaRequestMux0Reserved15 = 15|0x100U, 
    kDmaRequestMux0SPI0Rx = 16|0x100U, 
    kDmaRequestMux0SPI0Tx = 17|0x100U, 
    kDmaRequestMux0SPI1Rx = 18|0x100U, 
    kDmaRequestMux0SPI1Tx = 19|0x100U, 
    kDmaRequestMux0Reserved20 = 20|0x100U, 
    kDmaRequestMux0Reserved21 = 21|0x100U, 
    kDmaRequestMux0I2C0 = 22|0x100U, 
    kDmaRequestMux0I2C1 = 23|0x100U, 
    kDmaRequestMux0TPM0Channel0 = 24|0x100U, 
    kDmaRequestMux0TPM0Channel1 = 25|0x100U, 
    kDmaRequestMux0TPM0Channel2 = 26|0x100U, 
    kDmaRequestMux0TPM0Channel3 = 27|0x100U, 
    kDmaRequestMux0TPM0Channel4 = 28|0x100U, 
    kDmaRequestMux0TPM0Channel5 = 29|0x100U, 
    kDmaRequestMux0Reserved30 = 30|0x100U, 
    kDmaRequestMux0Reserved31 = 31|0x100U, 
    kDmaRequestMux0TPM1Channel0 = 32|0x100U, 
    kDmaRequestMux0TPM1Channel1 = 33|0x100U, 
    kDmaRequestMux0TPM2Channel0 = 34|0x100U, 
    kDmaRequestMux0TPM2Channel1 = 35|0x100U, 
    kDmaRequestMux0Reserved36 = 36|0x100U, 
    kDmaRequestMux0Reserved37 = 37|0x100U, 
    kDmaRequestMux0Reserved38 = 38|0x100U, 
    kDmaRequestMux0Reserved39 = 39|0x100U, 
    kDmaRequestMux0ADC0 = 40|0x100U, 
    kDmaRequestMux0Reserved41 = 41|0x100U, 
    kDmaRequestMux0CMP0 = 42|0x100U, 
    kDmaRequestMux0Reserved43 = 43|0x100U, 
    kDmaRequestMux0Reserved44 = 44|0x100U, 
    kDmaRequestMux0DAC0 = 45|0x100U, 
    kDmaRequestMux0Reserved46 = 46|0x100U, 
    kDmaRequestMux0Reserved47 = 47|0x100U, 
    kDmaRequestMux0Reserved48 = 48|0x100U, 
    kDmaRequestMux0PortA = 49|0x100U, 
    kDmaRequestMux0Reserved50 = 50|0x100U, 
    kDmaRequestMux0PortC = 51|0x100U, 
    kDmaRequestMux0PortD = 52|0x100U, 
    kDmaRequestMux0Reserved53 = 53|0x100U, 
    kDmaRequestMux0TPM0Overflow = 54|0x100U, 
    kDmaRequestMux0TPM1Overflow = 55|0x100U, 
    kDmaRequestMux0TPM2Overflow = 56|0x100U, 
    kDmaRequestMux0TSI0 = 57|0x100U, 
    kDmaRequestMux0Reserved58 = 58|0x100U, 
    kDmaRequestMux0Reserved59 = 59|0x100U, 
    kDmaRequestMux0AlwaysOn60 = 60|0x100U, 
    kDmaRequestMux0AlwaysOn61 = 61|0x100U, 
    kDmaRequestMux0AlwaysOn62 = 62|0x100U, 
    kDmaRequestMux0AlwaysOn63 = 63|0x100U, 
#else
    #error "No valid CPU defined!"
#endif
} dma_request_source_t;



#endif 

/*******************************************************************************
 * EOF
 ******************************************************************************/

